/*************************************************************************
 * math stuff
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 ************************************************************************/


#include "math.h"


ANGLE ArcTanTable[256];


/******************************************************************
 * get an approximate distance between two 2D points, using quicky
 * code
 *
 * IN:		the two points
 *
 * RETURNS:	approx. distance between the two points
 ******************************************************************/
s24f8 s24f8vect2D_approxDist (s24f8_vect2D *pA, s24f8_vect2D *pB)
{
	s24f8 dx = pA->x - pB->x;
	s24f8 dy = pA->y - pB->y;

	return((dx + dy) - (MIN(dx, dy)>>1));
}



/********************************************
 * get the angle between two points
 *
 * IN:		pA, pB = 2 points
 *
 * RETURNS:	angle
 *******************************************/
ANGLE Math_GetAngleFromPoints (s24f8_vect2D *pA, s24f8_vect2D *pB)
{
s24f8 x, y;
s24f8 val;
char xNeg, yNeg;
ANGLE out;

	// get the vector to the target point
	x = pB->x - pA->x;
	y = pB->y - pA->y;

	// make sure the values we are using are both positive (preserving their sign for later)
	if (x < 0)
	{
		xNeg = 1;
		x = -x;
	}
	else
	{
		xNeg = 0;
	}

	if (y < 0)
	{
		yNeg = 1;
		y = -y;
	}
	else
	{
		yNeg = 0;
	}

	// do the divide, but make sure it always turns out less than 0 (by doing either x/y or y/x)
	if (x > y)
	{
		val = s24f8_div(y, x);

		if (val >= int_to_s24f8(1))
		{	// NOTE: should never be > 1.0, but could be ==
			out = ANG_45;
		}
		else
		{
			out = ANG_90 - ArcTanTable[val];
		}
	}
	else if (y != 0)
	{
		val = s24f8_div(x, y);

		if (val == int_to_s24f8(1))
		{	// NOTE: should never be > 1.0, but could be ==
			out = ANG_45;
		}
		else
		{
			out = ArcTanTable[val];
		}
	}
	else
	{	// both x and y are 0... return an arbitrary direction
		return(0);
	}

	// orient the angle into the proper quadrant of the circle
	if (yNeg)
	{
		if (xNeg)
		{
			out = ANG_360 - out;
		}
	}
	else
	{
		if (xNeg)
		{
			out += ANG_180;
		}
		else
		{
			out = ANG_180 - out;
		}
	}

	return(out);
}


u32 randomLastValue = 1;

/******************************************************************
 * re-seed the random number generator
 *****************************************************************/
void Random_ReSeed (u32 seed)
{
	randomLastValue = seed;
}

/******************************************************************
 * get a random number
 *****************************************************************/
u32 Random_Get32 (void)
{
u32 r;

	r  = ( randomLastValue >> 1 ) | ( ( randomLastValue & 1 ) << 31 );		// ROR #1
	r ^= randomLastValue;
	randomLastValue -= r;

	return (randomLastValue);
}



// sine and cosine tables
s24f8 SIN_TABLE[256] =
{
	0x00000000, // 0 (angle ~ 0.0000 degrees)
	0x00000006, // 1 (angle ~ 1.4063 degrees)
	0x0000000C, // 2 (angle ~ 2.8125 degrees)
	0x00000012, // 3 (angle ~ 4.2188 degrees)
	0x00000019, // 4 (angle ~ 5.6250 degrees)
	0x0000001F, // 5 (angle ~ 7.0313 degrees)
	0x00000025, // 6 (angle ~ 8.4375 degrees)
	0x0000002B, // 7 (angle ~ 9.8438 degrees)
	0x00000031, // 8 (angle ~ 11.2500 degrees)
	0x00000038, // 9 (angle ~ 12.6563 degrees)
	0x0000003E, // 10 (angle ~ 14.0625 degrees)
	0x00000044, // 11 (angle ~ 15.4688 degrees)
	0x0000004A, // 12 (angle ~ 16.8750 degrees)
	0x00000050, // 13 (angle ~ 18.2813 degrees)
	0x00000056, // 14 (angle ~ 19.6875 degrees)
	0x0000005C, // 15 (angle ~ 21.0938 degrees)
	0x00000061, // 16 (angle ~ 22.5000 degrees)
	0x00000067, // 17 (angle ~ 23.9063 degrees)
	0x0000006D, // 18 (angle ~ 25.3125 degrees)
	0x00000073, // 19 (angle ~ 26.7188 degrees)
	0x00000078, // 20 (angle ~ 28.1250 degrees)
	0x0000007E, // 21 (angle ~ 29.5313 degrees)
	0x00000083, // 22 (angle ~ 30.9375 degrees)
	0x00000088, // 23 (angle ~ 32.3438 degrees)
	0x0000008E, // 24 (angle ~ 33.7500 degrees)
	0x00000093, // 25 (angle ~ 35.1563 degrees)
	0x00000098, // 26 (angle ~ 36.5625 degrees)
	0x0000009D, // 27 (angle ~ 37.9688 degrees)
	0x000000A2, // 28 (angle ~ 39.3750 degrees)
	0x000000A7, // 29 (angle ~ 40.7813 degrees)
	0x000000AB, // 30 (angle ~ 42.1875 degrees)
	0x000000B0, // 31 (angle ~ 43.5938 degrees)
	0x000000B5, // 32 (angle ~ 45.0000 degrees)
	0x000000B9, // 33 (angle ~ 46.4063 degrees)
	0x000000BD, // 34 (angle ~ 47.8125 degrees)
	0x000000C1, // 35 (angle ~ 49.2188 degrees)
	0x000000C5, // 36 (angle ~ 50.6250 degrees)
	0x000000C9, // 37 (angle ~ 52.0313 degrees)
	0x000000CD, // 38 (angle ~ 53.4375 degrees)
	0x000000D1, // 39 (angle ~ 54.8438 degrees)
	0x000000D4, // 40 (angle ~ 56.2500 degrees)
	0x000000D8, // 41 (angle ~ 57.6563 degrees)
	0x000000DB, // 42 (angle ~ 59.0625 degrees)
	0x000000DE, // 43 (angle ~ 60.4688 degrees)
	0x000000E1, // 44 (angle ~ 61.8750 degrees)
	0x000000E4, // 45 (angle ~ 63.2813 degrees)
	0x000000E7, // 46 (angle ~ 64.6875 degrees)
	0x000000EA, // 47 (angle ~ 66.0938 degrees)
	0x000000EC, // 48 (angle ~ 67.5000 degrees)
	0x000000EE, // 49 (angle ~ 68.9063 degrees)
	0x000000F1, // 50 (angle ~ 70.3125 degrees)
	0x000000F3, // 51 (angle ~ 71.7188 degrees)
	0x000000F4, // 52 (angle ~ 73.1250 degrees)
	0x000000F6, // 53 (angle ~ 74.5313 degrees)
	0x000000F8, // 54 (angle ~ 75.9375 degrees)
	0x000000F9, // 55 (angle ~ 77.3438 degrees)
	0x000000FB, // 56 (angle ~ 78.7500 degrees)
	0x000000FC, // 57 (angle ~ 80.1563 degrees)
	0x000000FD, // 58 (angle ~ 81.5625 degrees)
	0x000000FE, // 59 (angle ~ 82.9688 degrees)
	0x000000FE, // 60 (angle ~ 84.3750 degrees)
	0x000000FF, // 61 (angle ~ 85.7813 degrees)
	0x000000FF, // 62 (angle ~ 87.1875 degrees)
	0x000000FF, // 63 (angle ~ 88.5938 degrees)
	0x000000FF, // 64 (angle ~ 90.0000 degrees)
	0x000000FF, // 65 (angle ~ 91.4063 degrees)
	0x000000FF, // 66 (angle ~ 92.8125 degrees)
	0x000000FF, // 67 (angle ~ 94.2188 degrees)
	0x000000FE, // 68 (angle ~ 95.6250 degrees)
	0x000000FE, // 69 (angle ~ 97.0313 degrees)
	0x000000FD, // 70 (angle ~ 98.4375 degrees)
	0x000000FC, // 71 (angle ~ 99.8438 degrees)
	0x000000FB, // 72 (angle ~ 101.2500 degrees)
	0x000000F9, // 73 (angle ~ 102.6563 degrees)
	0x000000F8, // 74 (angle ~ 104.0625 degrees)
	0x000000F6, // 75 (angle ~ 105.4688 degrees)
	0x000000F4, // 76 (angle ~ 106.8750 degrees)
	0x000000F3, // 77 (angle ~ 108.2813 degrees)
	0x000000F1, // 78 (angle ~ 109.6875 degrees)
	0x000000EE, // 79 (angle ~ 111.0938 degrees)
	0x000000EC, // 80 (angle ~ 112.5000 degrees)
	0x000000EA, // 81 (angle ~ 113.9063 degrees)
	0x000000E7, // 82 (angle ~ 115.3125 degrees)
	0x000000E4, // 83 (angle ~ 116.7188 degrees)
	0x000000E1, // 84 (angle ~ 118.1250 degrees)
	0x000000DE, // 85 (angle ~ 119.5313 degrees)
	0x000000DB, // 86 (angle ~ 120.9375 degrees)
	0x000000D8, // 87 (angle ~ 122.3438 degrees)
	0x000000D4, // 88 (angle ~ 123.7500 degrees)
	0x000000D1, // 89 (angle ~ 125.1563 degrees)
	0x000000CD, // 90 (angle ~ 126.5625 degrees)
	0x000000C9, // 91 (angle ~ 127.9688 degrees)
	0x000000C5, // 92 (angle ~ 129.3750 degrees)
	0x000000C1, // 93 (angle ~ 130.7813 degrees)
	0x000000BD, // 94 (angle ~ 132.1875 degrees)
	0x000000B9, // 95 (angle ~ 133.5938 degrees)
	0x000000B5, // 96 (angle ~ 135.0000 degrees)
	0x000000B0, // 97 (angle ~ 136.4063 degrees)
	0x000000AB, // 98 (angle ~ 137.8125 degrees)
	0x000000A7, // 99 (angle ~ 139.2188 degrees)
	0x000000A2, // 100 (angle ~ 140.6250 degrees)
	0x0000009D, // 101 (angle ~ 142.0313 degrees)
	0x00000098, // 102 (angle ~ 143.4375 degrees)
	0x00000093, // 103 (angle ~ 144.8438 degrees)
	0x0000008E, // 104 (angle ~ 146.2500 degrees)
	0x00000088, // 105 (angle ~ 147.6563 degrees)
	0x00000083, // 106 (angle ~ 149.0625 degrees)
	0x0000007E, // 107 (angle ~ 150.4688 degrees)
	0x00000078, // 108 (angle ~ 151.8750 degrees)
	0x00000073, // 109 (angle ~ 153.2813 degrees)
	0x0000006D, // 110 (angle ~ 154.6875 degrees)
	0x00000067, // 111 (angle ~ 156.0938 degrees)
	0x00000061, // 112 (angle ~ 157.5000 degrees)
	0x0000005C, // 113 (angle ~ 158.9063 degrees)
	0x00000056, // 114 (angle ~ 160.3125 degrees)
	0x00000050, // 115 (angle ~ 161.7188 degrees)
	0x0000004A, // 116 (angle ~ 163.1250 degrees)
	0x00000044, // 117 (angle ~ 164.5313 degrees)
	0x0000003E, // 118 (angle ~ 165.9375 degrees)
	0x00000038, // 119 (angle ~ 167.3438 degrees)
	0x00000031, // 120 (angle ~ 168.7500 degrees)
	0x0000002B, // 121 (angle ~ 170.1563 degrees)
	0x00000025, // 122 (angle ~ 171.5625 degrees)
	0x0000001F, // 123 (angle ~ 172.9688 degrees)
	0x00000019, // 124 (angle ~ 174.3750 degrees)
	0x00000012, // 125 (angle ~ 175.7813 degrees)
	0x0000000C, // 126 (angle ~ 177.1875 degrees)
	0x00000006, // 127 (angle ~ 178.5938 degrees)
	0x00000000, // 128 (angle ~ 180.0000 degrees)
	0xFFFFFFFA, // 129 (angle ~ 181.4063 degrees)
	0xFFFFFFF4, // 130 (angle ~ 182.8125 degrees)
	0xFFFFFFEE, // 131 (angle ~ 184.2188 degrees)
	0xFFFFFFE7, // 132 (angle ~ 185.6250 degrees)
	0xFFFFFFE1, // 133 (angle ~ 187.0313 degrees)
	0xFFFFFFDB, // 134 (angle ~ 188.4375 degrees)
	0xFFFFFFD5, // 135 (angle ~ 189.8438 degrees)
	0xFFFFFFCF, // 136 (angle ~ 191.2500 degrees)
	0xFFFFFFC8, // 137 (angle ~ 192.6563 degrees)
	0xFFFFFFC2, // 138 (angle ~ 194.0625 degrees)
	0xFFFFFFBC, // 139 (angle ~ 195.4688 degrees)
	0xFFFFFFB6, // 140 (angle ~ 196.8750 degrees)
	0xFFFFFFB0, // 141 (angle ~ 198.2813 degrees)
	0xFFFFFFAA, // 142 (angle ~ 199.6875 degrees)
	0xFFFFFFA4, // 143 (angle ~ 201.0938 degrees)
	0xFFFFFF9F, // 144 (angle ~ 202.5000 degrees)
	0xFFFFFF99, // 145 (angle ~ 203.9063 degrees)
	0xFFFFFF93, // 146 (angle ~ 205.3125 degrees)
	0xFFFFFF8D, // 147 (angle ~ 206.7188 degrees)
	0xFFFFFF88, // 148 (angle ~ 208.1250 degrees)
	0xFFFFFF82, // 149 (angle ~ 209.5313 degrees)
	0xFFFFFF7D, // 150 (angle ~ 210.9375 degrees)
	0xFFFFFF78, // 151 (angle ~ 212.3438 degrees)
	0xFFFFFF72, // 152 (angle ~ 213.7500 degrees)
	0xFFFFFF6D, // 153 (angle ~ 215.1563 degrees)
	0xFFFFFF68, // 154 (angle ~ 216.5625 degrees)
	0xFFFFFF63, // 155 (angle ~ 217.9688 degrees)
	0xFFFFFF5E, // 156 (angle ~ 219.3750 degrees)
	0xFFFFFF59, // 157 (angle ~ 220.7813 degrees)
	0xFFFFFF55, // 158 (angle ~ 222.1875 degrees)
	0xFFFFFF50, // 159 (angle ~ 223.5938 degrees)
	0xFFFFFF4B, // 160 (angle ~ 225.0000 degrees)
	0xFFFFFF47, // 161 (angle ~ 226.4063 degrees)
	0xFFFFFF43, // 162 (angle ~ 227.8125 degrees)
	0xFFFFFF3F, // 163 (angle ~ 229.2188 degrees)
	0xFFFFFF3B, // 164 (angle ~ 230.6250 degrees)
	0xFFFFFF37, // 165 (angle ~ 232.0313 degrees)
	0xFFFFFF33, // 166 (angle ~ 233.4375 degrees)
	0xFFFFFF2F, // 167 (angle ~ 234.8438 degrees)
	0xFFFFFF2C, // 168 (angle ~ 236.2500 degrees)
	0xFFFFFF28, // 169 (angle ~ 237.6563 degrees)
	0xFFFFFF25, // 170 (angle ~ 239.0625 degrees)
	0xFFFFFF22, // 171 (angle ~ 240.4688 degrees)
	0xFFFFFF1F, // 172 (angle ~ 241.8750 degrees)
	0xFFFFFF1C, // 173 (angle ~ 243.2813 degrees)
	0xFFFFFF19, // 174 (angle ~ 244.6875 degrees)
	0xFFFFFF16, // 175 (angle ~ 246.0938 degrees)
	0xFFFFFF14, // 176 (angle ~ 247.5000 degrees)
	0xFFFFFF12, // 177 (angle ~ 248.9063 degrees)
	0xFFFFFF0F, // 178 (angle ~ 250.3125 degrees)
	0xFFFFFF0D, // 179 (angle ~ 251.7188 degrees)
	0xFFFFFF0C, // 180 (angle ~ 253.1250 degrees)
	0xFFFFFF0A, // 181 (angle ~ 254.5313 degrees)
	0xFFFFFF08, // 182 (angle ~ 255.9375 degrees)
	0xFFFFFF07, // 183 (angle ~ 257.3438 degrees)
	0xFFFFFF05, // 184 (angle ~ 258.7500 degrees)
	0xFFFFFF04, // 185 (angle ~ 260.1563 degrees)
	0xFFFFFF03, // 186 (angle ~ 261.5625 degrees)
	0xFFFFFF02, // 187 (angle ~ 262.9688 degrees)
	0xFFFFFF02, // 188 (angle ~ 264.3750 degrees)
	0xFFFFFF01, // 189 (angle ~ 265.7813 degrees)
	0xFFFFFF01, // 190 (angle ~ 267.1875 degrees)
	0xFFFFFF01, // 191 (angle ~ 268.5938 degrees)
	0xFFFFFF01, // 192 (angle ~ 270.0000 degrees)
	0xFFFFFF01, // 193 (angle ~ 271.4063 degrees)
	0xFFFFFF01, // 194 (angle ~ 272.8125 degrees)
	0xFFFFFF01, // 195 (angle ~ 274.2188 degrees)
	0xFFFFFF02, // 196 (angle ~ 275.6250 degrees)
	0xFFFFFF02, // 197 (angle ~ 277.0313 degrees)
	0xFFFFFF03, // 198 (angle ~ 278.4375 degrees)
	0xFFFFFF04, // 199 (angle ~ 279.8438 degrees)
	0xFFFFFF05, // 200 (angle ~ 281.2500 degrees)
	0xFFFFFF07, // 201 (angle ~ 282.6563 degrees)
	0xFFFFFF08, // 202 (angle ~ 284.0625 degrees)
	0xFFFFFF0A, // 203 (angle ~ 285.4688 degrees)
	0xFFFFFF0C, // 204 (angle ~ 286.8750 degrees)
	0xFFFFFF0D, // 205 (angle ~ 288.2813 degrees)
	0xFFFFFF0F, // 206 (angle ~ 289.6875 degrees)
	0xFFFFFF12, // 207 (angle ~ 291.0938 degrees)
	0xFFFFFF14, // 208 (angle ~ 292.5000 degrees)
	0xFFFFFF16, // 209 (angle ~ 293.9063 degrees)
	0xFFFFFF19, // 210 (angle ~ 295.3125 degrees)
	0xFFFFFF1C, // 211 (angle ~ 296.7188 degrees)
	0xFFFFFF1F, // 212 (angle ~ 298.1250 degrees)
	0xFFFFFF22, // 213 (angle ~ 299.5313 degrees)
	0xFFFFFF25, // 214 (angle ~ 300.9375 degrees)
	0xFFFFFF28, // 215 (angle ~ 302.3438 degrees)
	0xFFFFFF2C, // 216 (angle ~ 303.7500 degrees)
	0xFFFFFF2F, // 217 (angle ~ 305.1563 degrees)
	0xFFFFFF33, // 218 (angle ~ 306.5625 degrees)
	0xFFFFFF37, // 219 (angle ~ 307.9688 degrees)
	0xFFFFFF3B, // 220 (angle ~ 309.3750 degrees)
	0xFFFFFF3F, // 221 (angle ~ 310.7813 degrees)
	0xFFFFFF43, // 222 (angle ~ 312.1875 degrees)
	0xFFFFFF47, // 223 (angle ~ 313.5938 degrees)
	0xFFFFFF4B, // 224 (angle ~ 315.0000 degrees)
	0xFFFFFF50, // 225 (angle ~ 316.4063 degrees)
	0xFFFFFF55, // 226 (angle ~ 317.8125 degrees)
	0xFFFFFF59, // 227 (angle ~ 319.2188 degrees)
	0xFFFFFF5E, // 228 (angle ~ 320.6250 degrees)
	0xFFFFFF63, // 229 (angle ~ 322.0313 degrees)
	0xFFFFFF68, // 230 (angle ~ 323.4375 degrees)
	0xFFFFFF6D, // 231 (angle ~ 324.8438 degrees)
	0xFFFFFF72, // 232 (angle ~ 326.2500 degrees)
	0xFFFFFF78, // 233 (angle ~ 327.6563 degrees)
	0xFFFFFF7D, // 234 (angle ~ 329.0625 degrees)
	0xFFFFFF82, // 235 (angle ~ 330.4688 degrees)
	0xFFFFFF88, // 236 (angle ~ 331.8750 degrees)
	0xFFFFFF8D, // 237 (angle ~ 333.2813 degrees)
	0xFFFFFF93, // 238 (angle ~ 334.6875 degrees)
	0xFFFFFF99, // 239 (angle ~ 336.0938 degrees)
	0xFFFFFF9F, // 240 (angle ~ 337.5000 degrees)
	0xFFFFFFA4, // 241 (angle ~ 338.9063 degrees)
	0xFFFFFFAA, // 242 (angle ~ 340.3125 degrees)
	0xFFFFFFB0, // 243 (angle ~ 341.7188 degrees)
	0xFFFFFFB6, // 244 (angle ~ 343.1250 degrees)
	0xFFFFFFBC, // 245 (angle ~ 344.5313 degrees)
	0xFFFFFFC2, // 246 (angle ~ 345.9375 degrees)
	0xFFFFFFC8, // 247 (angle ~ 347.3438 degrees)
	0xFFFFFFCF, // 248 (angle ~ 348.7500 degrees)
	0xFFFFFFD5, // 249 (angle ~ 350.1563 degrees)
	0xFFFFFFDB, // 250 (angle ~ 351.5625 degrees)
	0xFFFFFFE1, // 251 (angle ~ 352.9688 degrees)
	0xFFFFFFE7, // 252 (angle ~ 354.3750 degrees)
	0xFFFFFFEE, // 253 (angle ~ 355.7813 degrees)
	0xFFFFFFF4, // 254 (angle ~ 357.1875 degrees)
	0xFFFFFFFA, // 255 (angle ~ 358.5938 degrees)
};


s24f8 COS_TABLE[256] =
{
	0x00000100, // 0 (angle ~ 0.0000 degrees)
	0x000000FF, // 1 (angle ~ 1.4063 degrees)
	0x000000FF, // 2 (angle ~ 2.8125 degrees)
	0x000000FF, // 3 (angle ~ 4.2188 degrees)
	0x000000FE, // 4 (angle ~ 5.6250 degrees)
	0x000000FE, // 5 (angle ~ 7.0313 degrees)
	0x000000FD, // 6 (angle ~ 8.4375 degrees)
	0x000000FC, // 7 (angle ~ 9.8438 degrees)
	0x000000FB, // 8 (angle ~ 11.2500 degrees)
	0x000000F9, // 9 (angle ~ 12.6563 degrees)
	0x000000F8, // 10 (angle ~ 14.0625 degrees)
	0x000000F6, // 11 (angle ~ 15.4688 degrees)
	0x000000F4, // 12 (angle ~ 16.8750 degrees)
	0x000000F3, // 13 (angle ~ 18.2813 degrees)
	0x000000F1, // 14 (angle ~ 19.6875 degrees)
	0x000000EE, // 15 (angle ~ 21.0938 degrees)
	0x000000EC, // 16 (angle ~ 22.5000 degrees)
	0x000000EA, // 17 (angle ~ 23.9063 degrees)
	0x000000E7, // 18 (angle ~ 25.3125 degrees)
	0x000000E4, // 19 (angle ~ 26.7188 degrees)
	0x000000E1, // 20 (angle ~ 28.1250 degrees)
	0x000000DE, // 21 (angle ~ 29.5313 degrees)
	0x000000DB, // 22 (angle ~ 30.9375 degrees)
	0x000000D8, // 23 (angle ~ 32.3438 degrees)
	0x000000D4, // 24 (angle ~ 33.7500 degrees)
	0x000000D1, // 25 (angle ~ 35.1563 degrees)
	0x000000CD, // 26 (angle ~ 36.5625 degrees)
	0x000000C9, // 27 (angle ~ 37.9688 degrees)
	0x000000C5, // 28 (angle ~ 39.3750 degrees)
	0x000000C1, // 29 (angle ~ 40.7813 degrees)
	0x000000BD, // 30 (angle ~ 42.1875 degrees)
	0x000000B9, // 31 (angle ~ 43.5938 degrees)
	0x000000B5, // 32 (angle ~ 45.0000 degrees)
	0x000000B0, // 33 (angle ~ 46.4063 degrees)
	0x000000AB, // 34 (angle ~ 47.8125 degrees)
	0x000000A7, // 35 (angle ~ 49.2188 degrees)
	0x000000A2, // 36 (angle ~ 50.6250 degrees)
	0x0000009D, // 37 (angle ~ 52.0313 degrees)
	0x00000098, // 38 (angle ~ 53.4375 degrees)
	0x00000093, // 39 (angle ~ 54.8438 degrees)
	0x0000008E, // 40 (angle ~ 56.2500 degrees)
	0x00000088, // 41 (angle ~ 57.6563 degrees)
	0x00000083, // 42 (angle ~ 59.0625 degrees)
	0x0000007E, // 43 (angle ~ 60.4688 degrees)
	0x00000078, // 44 (angle ~ 61.8750 degrees)
	0x00000073, // 45 (angle ~ 63.2813 degrees)
	0x0000006D, // 46 (angle ~ 64.6875 degrees)
	0x00000067, // 47 (angle ~ 66.0938 degrees)
	0x00000061, // 48 (angle ~ 67.5000 degrees)
	0x0000005C, // 49 (angle ~ 68.9063 degrees)
	0x00000056, // 50 (angle ~ 70.3125 degrees)
	0x00000050, // 51 (angle ~ 71.7188 degrees)
	0x0000004A, // 52 (angle ~ 73.1250 degrees)
	0x00000044, // 53 (angle ~ 74.5313 degrees)
	0x0000003E, // 54 (angle ~ 75.9375 degrees)
	0x00000038, // 55 (angle ~ 77.3438 degrees)
	0x00000031, // 56 (angle ~ 78.7500 degrees)
	0x0000002B, // 57 (angle ~ 80.1563 degrees)
	0x00000025, // 58 (angle ~ 81.5625 degrees)
	0x0000001F, // 59 (angle ~ 82.9688 degrees)
	0x00000019, // 60 (angle ~ 84.3750 degrees)
	0x00000012, // 61 (angle ~ 85.7813 degrees)
	0x0000000C, // 62 (angle ~ 87.1875 degrees)
	0x00000006, // 63 (angle ~ 88.5938 degrees)
	0x00000000, // 64 (angle ~ 90.0000 degrees)
	0xFFFFFFFA, // 65 (angle ~ 91.4063 degrees)
	0xFFFFFFF4, // 66 (angle ~ 92.8125 degrees)
	0xFFFFFFEE, // 67 (angle ~ 94.2188 degrees)
	0xFFFFFFE7, // 68 (angle ~ 95.6250 degrees)
	0xFFFFFFE1, // 69 (angle ~ 97.0313 degrees)
	0xFFFFFFDB, // 70 (angle ~ 98.4375 degrees)
	0xFFFFFFD5, // 71 (angle ~ 99.8438 degrees)
	0xFFFFFFCF, // 72 (angle ~ 101.2500 degrees)
	0xFFFFFFC8, // 73 (angle ~ 102.6563 degrees)
	0xFFFFFFC2, // 74 (angle ~ 104.0625 degrees)
	0xFFFFFFBC, // 75 (angle ~ 105.4688 degrees)
	0xFFFFFFB6, // 76 (angle ~ 106.8750 degrees)
	0xFFFFFFB0, // 77 (angle ~ 108.2813 degrees)
	0xFFFFFFAA, // 78 (angle ~ 109.6875 degrees)
	0xFFFFFFA4, // 79 (angle ~ 111.0938 degrees)
	0xFFFFFF9F, // 80 (angle ~ 112.5000 degrees)
	0xFFFFFF99, // 81 (angle ~ 113.9063 degrees)
	0xFFFFFF93, // 82 (angle ~ 115.3125 degrees)
	0xFFFFFF8D, // 83 (angle ~ 116.7188 degrees)
	0xFFFFFF88, // 84 (angle ~ 118.1250 degrees)
	0xFFFFFF82, // 85 (angle ~ 119.5313 degrees)
	0xFFFFFF7D, // 86 (angle ~ 120.9375 degrees)
	0xFFFFFF78, // 87 (angle ~ 122.3438 degrees)
	0xFFFFFF72, // 88 (angle ~ 123.7500 degrees)
	0xFFFFFF6D, // 89 (angle ~ 125.1563 degrees)
	0xFFFFFF68, // 90 (angle ~ 126.5625 degrees)
	0xFFFFFF63, // 91 (angle ~ 127.9688 degrees)
	0xFFFFFF5E, // 92 (angle ~ 129.3750 degrees)
	0xFFFFFF59, // 93 (angle ~ 130.7813 degrees)
	0xFFFFFF55, // 94 (angle ~ 132.1875 degrees)
	0xFFFFFF50, // 95 (angle ~ 133.5938 degrees)
	0xFFFFFF4B, // 96 (angle ~ 135.0000 degrees)
	0xFFFFFF47, // 97 (angle ~ 136.4063 degrees)
	0xFFFFFF43, // 98 (angle ~ 137.8125 degrees)
	0xFFFFFF3F, // 99 (angle ~ 139.2188 degrees)
	0xFFFFFF3B, // 100 (angle ~ 140.6250 degrees)
	0xFFFFFF37, // 101 (angle ~ 142.0313 degrees)
	0xFFFFFF33, // 102 (angle ~ 143.4375 degrees)
	0xFFFFFF2F, // 103 (angle ~ 144.8438 degrees)
	0xFFFFFF2C, // 104 (angle ~ 146.2500 degrees)
	0xFFFFFF28, // 105 (angle ~ 147.6563 degrees)
	0xFFFFFF25, // 106 (angle ~ 149.0625 degrees)
	0xFFFFFF22, // 107 (angle ~ 150.4688 degrees)
	0xFFFFFF1F, // 108 (angle ~ 151.8750 degrees)
	0xFFFFFF1C, // 109 (angle ~ 153.2813 degrees)
	0xFFFFFF19, // 110 (angle ~ 154.6875 degrees)
	0xFFFFFF16, // 111 (angle ~ 156.0938 degrees)
	0xFFFFFF14, // 112 (angle ~ 157.5000 degrees)
	0xFFFFFF12, // 113 (angle ~ 158.9063 degrees)
	0xFFFFFF0F, // 114 (angle ~ 160.3125 degrees)
	0xFFFFFF0D, // 115 (angle ~ 161.7188 degrees)
	0xFFFFFF0C, // 116 (angle ~ 163.1250 degrees)
	0xFFFFFF0A, // 117 (angle ~ 164.5313 degrees)
	0xFFFFFF08, // 118 (angle ~ 165.9375 degrees)
	0xFFFFFF07, // 119 (angle ~ 167.3438 degrees)
	0xFFFFFF05, // 120 (angle ~ 168.7500 degrees)
	0xFFFFFF04, // 121 (angle ~ 170.1563 degrees)
	0xFFFFFF03, // 122 (angle ~ 171.5625 degrees)
	0xFFFFFF02, // 123 (angle ~ 172.9688 degrees)
	0xFFFFFF02, // 124 (angle ~ 174.3750 degrees)
	0xFFFFFF01, // 125 (angle ~ 175.7813 degrees)
	0xFFFFFF01, // 126 (angle ~ 177.1875 degrees)
	0xFFFFFF01, // 127 (angle ~ 178.5938 degrees)
	0xFFFFFF01, // 128 (angle ~ 180.0000 degrees)
	0xFFFFFF01, // 129 (angle ~ 181.4063 degrees)
	0xFFFFFF01, // 130 (angle ~ 182.8125 degrees)
	0xFFFFFF01, // 131 (angle ~ 184.2188 degrees)
	0xFFFFFF02, // 132 (angle ~ 185.6250 degrees)
	0xFFFFFF02, // 133 (angle ~ 187.0313 degrees)
	0xFFFFFF03, // 134 (angle ~ 188.4375 degrees)
	0xFFFFFF04, // 135 (angle ~ 189.8438 degrees)
	0xFFFFFF05, // 136 (angle ~ 191.2500 degrees)
	0xFFFFFF07, // 137 (angle ~ 192.6563 degrees)
	0xFFFFFF08, // 138 (angle ~ 194.0625 degrees)
	0xFFFFFF0A, // 139 (angle ~ 195.4688 degrees)
	0xFFFFFF0C, // 140 (angle ~ 196.8750 degrees)
	0xFFFFFF0D, // 141 (angle ~ 198.2813 degrees)
	0xFFFFFF0F, // 142 (angle ~ 199.6875 degrees)
	0xFFFFFF12, // 143 (angle ~ 201.0938 degrees)
	0xFFFFFF14, // 144 (angle ~ 202.5000 degrees)
	0xFFFFFF16, // 145 (angle ~ 203.9063 degrees)
	0xFFFFFF19, // 146 (angle ~ 205.3125 degrees)
	0xFFFFFF1C, // 147 (angle ~ 206.7188 degrees)
	0xFFFFFF1F, // 148 (angle ~ 208.1250 degrees)
	0xFFFFFF22, // 149 (angle ~ 209.5313 degrees)
	0xFFFFFF25, // 150 (angle ~ 210.9375 degrees)
	0xFFFFFF28, // 151 (angle ~ 212.3438 degrees)
	0xFFFFFF2C, // 152 (angle ~ 213.7500 degrees)
	0xFFFFFF2F, // 153 (angle ~ 215.1563 degrees)
	0xFFFFFF33, // 154 (angle ~ 216.5625 degrees)
	0xFFFFFF37, // 155 (angle ~ 217.9688 degrees)
	0xFFFFFF3B, // 156 (angle ~ 219.3750 degrees)
	0xFFFFFF3F, // 157 (angle ~ 220.7813 degrees)
	0xFFFFFF43, // 158 (angle ~ 222.1875 degrees)
	0xFFFFFF47, // 159 (angle ~ 223.5938 degrees)
	0xFFFFFF4B, // 160 (angle ~ 225.0000 degrees)
	0xFFFFFF50, // 161 (angle ~ 226.4063 degrees)
	0xFFFFFF55, // 162 (angle ~ 227.8125 degrees)
	0xFFFFFF59, // 163 (angle ~ 229.2188 degrees)
	0xFFFFFF5E, // 164 (angle ~ 230.6250 degrees)
	0xFFFFFF63, // 165 (angle ~ 232.0313 degrees)
	0xFFFFFF68, // 166 (angle ~ 233.4375 degrees)
	0xFFFFFF6D, // 167 (angle ~ 234.8438 degrees)
	0xFFFFFF72, // 168 (angle ~ 236.2500 degrees)
	0xFFFFFF78, // 169 (angle ~ 237.6563 degrees)
	0xFFFFFF7D, // 170 (angle ~ 239.0625 degrees)
	0xFFFFFF82, // 171 (angle ~ 240.4688 degrees)
	0xFFFFFF88, // 172 (angle ~ 241.8750 degrees)
	0xFFFFFF8D, // 173 (angle ~ 243.2813 degrees)
	0xFFFFFF93, // 174 (angle ~ 244.6875 degrees)
	0xFFFFFF99, // 175 (angle ~ 246.0938 degrees)
	0xFFFFFF9F, // 176 (angle ~ 247.5000 degrees)
	0xFFFFFFA4, // 177 (angle ~ 248.9063 degrees)
	0xFFFFFFAA, // 178 (angle ~ 250.3125 degrees)
	0xFFFFFFB0, // 179 (angle ~ 251.7188 degrees)
	0xFFFFFFB6, // 180 (angle ~ 253.1250 degrees)
	0xFFFFFFBC, // 181 (angle ~ 254.5313 degrees)
	0xFFFFFFC2, // 182 (angle ~ 255.9375 degrees)
	0xFFFFFFC8, // 183 (angle ~ 257.3438 degrees)
	0xFFFFFFCF, // 184 (angle ~ 258.7500 degrees)
	0xFFFFFFD5, // 185 (angle ~ 260.1563 degrees)
	0xFFFFFFDB, // 186 (angle ~ 261.5625 degrees)
	0xFFFFFFE1, // 187 (angle ~ 262.9688 degrees)
	0xFFFFFFE7, // 188 (angle ~ 264.3750 degrees)
	0xFFFFFFEE, // 189 (angle ~ 265.7813 degrees)
	0xFFFFFFF4, // 190 (angle ~ 267.1875 degrees)
	0xFFFFFFFA, // 191 (angle ~ 268.5938 degrees)
	0x00000000, // 192 (angle ~ 270.0000 degrees)
	0x00000006, // 193 (angle ~ 271.4063 degrees)
	0x0000000C, // 194 (angle ~ 272.8125 degrees)
	0x00000012, // 195 (angle ~ 274.2188 degrees)
	0x00000019, // 196 (angle ~ 275.6250 degrees)
	0x0000001F, // 197 (angle ~ 277.0313 degrees)
	0x00000025, // 198 (angle ~ 278.4375 degrees)
	0x0000002B, // 199 (angle ~ 279.8438 degrees)
	0x00000031, // 200 (angle ~ 281.2500 degrees)
	0x00000038, // 201 (angle ~ 282.6563 degrees)
	0x0000003E, // 202 (angle ~ 284.0625 degrees)
	0x00000044, // 203 (angle ~ 285.4688 degrees)
	0x0000004A, // 204 (angle ~ 286.8750 degrees)
	0x00000050, // 205 (angle ~ 288.2813 degrees)
	0x00000056, // 206 (angle ~ 289.6875 degrees)
	0x0000005C, // 207 (angle ~ 291.0938 degrees)
	0x00000061, // 208 (angle ~ 292.5000 degrees)
	0x00000067, // 209 (angle ~ 293.9063 degrees)
	0x0000006D, // 210 (angle ~ 295.3125 degrees)
	0x00000073, // 211 (angle ~ 296.7188 degrees)
	0x00000078, // 212 (angle ~ 298.1250 degrees)
	0x0000007E, // 213 (angle ~ 299.5313 degrees)
	0x00000083, // 214 (angle ~ 300.9375 degrees)
	0x00000088, // 215 (angle ~ 302.3438 degrees)
	0x0000008E, // 216 (angle ~ 303.7500 degrees)
	0x00000093, // 217 (angle ~ 305.1563 degrees)
	0x00000098, // 218 (angle ~ 306.5625 degrees)
	0x0000009D, // 219 (angle ~ 307.9688 degrees)
	0x000000A2, // 220 (angle ~ 309.3750 degrees)
	0x000000A7, // 221 (angle ~ 310.7813 degrees)
	0x000000AB, // 222 (angle ~ 312.1875 degrees)
	0x000000B0, // 223 (angle ~ 313.5938 degrees)
	0x000000B5, // 224 (angle ~ 315.0000 degrees)
	0x000000B9, // 225 (angle ~ 316.4063 degrees)
	0x000000BD, // 226 (angle ~ 317.8125 degrees)
	0x000000C1, // 227 (angle ~ 319.2188 degrees)
	0x000000C5, // 228 (angle ~ 320.6250 degrees)
	0x000000C9, // 229 (angle ~ 322.0313 degrees)
	0x000000CD, // 230 (angle ~ 323.4375 degrees)
	0x000000D1, // 231 (angle ~ 324.8438 degrees)
	0x000000D4, // 232 (angle ~ 326.2500 degrees)
	0x000000D8, // 233 (angle ~ 327.6563 degrees)
	0x000000DB, // 234 (angle ~ 329.0625 degrees)
	0x000000DE, // 235 (angle ~ 330.4688 degrees)
	0x000000E1, // 236 (angle ~ 331.8750 degrees)
	0x000000E4, // 237 (angle ~ 333.2813 degrees)
	0x000000E7, // 238 (angle ~ 334.6875 degrees)
	0x000000EA, // 239 (angle ~ 336.0938 degrees)
	0x000000EC, // 240 (angle ~ 337.5000 degrees)
	0x000000EE, // 241 (angle ~ 338.9063 degrees)
	0x000000F1, // 242 (angle ~ 340.3125 degrees)
	0x000000F3, // 243 (angle ~ 341.7188 degrees)
	0x000000F4, // 244 (angle ~ 343.1250 degrees)
	0x000000F6, // 245 (angle ~ 344.5313 degrees)
	0x000000F8, // 246 (angle ~ 345.9375 degrees)
	0x000000F9, // 247 (angle ~ 347.3438 degrees)
	0x000000FB, // 248 (angle ~ 348.7500 degrees)
	0x000000FC, // 249 (angle ~ 350.1563 degrees)
	0x000000FD, // 250 (angle ~ 351.5625 degrees)
	0x000000FE, // 251 (angle ~ 352.9688 degrees)
	0x000000FE, // 252 (angle ~ 354.3750 degrees)
	0x000000FF, // 253 (angle ~ 355.7813 degrees)
	0x000000FF, // 254 (angle ~ 357.1875 degrees)
	0x000000FF, // 255 (angle ~ 358.5938 degrees)
};


// arctan table
ANGLE ArcTanTable[256] =
{
	0,		// 0
	0,		// 1
	0,		// 2
	0,		// 3
	0,		// 4
	0,		// 5
	0,		// 6
	1,		// 7
	1,		// 8
	1,		// 9
	1,		// 10
	1,		// 11
	1,		// 12
	2,		// 13
	2,		// 14
	2,		// 15
	2,		// 16
	2,		// 17
	2,		// 18
	3,		// 19
	3,		// 20
	3,		// 21
	3,		// 22
	3,		// 23
	3,		// 24
	3,		// 25
	4,		// 26
	4,		// 27
	4,		// 28
	4,		// 29
	4,		// 30
	4,		// 31
	5,		// 32
	5,		// 33
	5,		// 34
	5,		// 35
	5,		// 36
	5,		// 37
	6,		// 38
	6,		// 39
	6,		// 40
	6,		// 41
	6,		// 42
	6,		// 43
	6,		// 44
	7,		// 45
	7,		// 46
	7,		// 47
	7,		// 48
	7,		// 49
	7,		// 50
	8,		// 51
	8,		// 52
	8,		// 53
	8,		// 54
	8,		// 55
	8,		// 56
	8,		// 57
	9,		// 58
	9,		// 59
	9,		// 60
	9,		// 61
	9,		// 62
	9,		// 63
	9,		// 64
	10,		// 65
	10,		// 66
	10,		// 67
	10,		// 68
	10,		// 69
	10,		// 70
	11,		// 71
	11,		// 72
	11,		// 73
	11,		// 74
	11,		// 75
	11,		// 76
	11,		// 77
	12,		// 78
	12,		// 79
	12,		// 80
	12,		// 81
	12,		// 82
	12,		// 83
	12,		// 84
	13,		// 85
	13,		// 86
	13,		// 87
	13,		// 88
	13,		// 89
	13,		// 90
	13,		// 91
	14,		// 92
	14,		// 93
	14,		// 94
	14,		// 95
	14,		// 96
	14,		// 97
	14,		// 98
	15,		// 99
	15,		// 100
	15,		// 101
	15,		// 102
	15,		// 103
	15,		// 104
	15,		// 105
	15,		// 106
	16,		// 107
	16,		// 108
	16,		// 109
	16,		// 110
	16,		// 111
	16,		// 112
	16,		// 113
	17,		// 114
	17,		// 115
	17,		// 116
	17,		// 117
	17,		// 118
	17,		// 119
	17,		// 120
	17,		// 121
	18,		// 122
	18,		// 123
	18,		// 124
	18,		// 125
	18,		// 126
	18,		// 127
	18,		// 128
	19,		// 129
	19,		// 130
	19,		// 131
	19,		// 132
	19,		// 133
	19,		// 134
	19,		// 135
	19,		// 136
	20,		// 137
	20,		// 138
	20,		// 139
	20,		// 140
	20,		// 141
	20,		// 142
	20,		// 143
	20,		// 144
	20,		// 145
	21,		// 146
	21,		// 147
	21,		// 148
	21,		// 149
	21,		// 150
	21,		// 151
	21,		// 152
	21,		// 153
	22,		// 154
	22,		// 155
	22,		// 156
	22,		// 157
	22,		// 158
	22,		// 159
	22,		// 160
	22,		// 161
	22,		// 162
	23,		// 163
	23,		// 164
	23,		// 165
	23,		// 166
	23,		// 167
	23,		// 168
	23,		// 169
	23,		// 170
	23,		// 171
	24,		// 172
	24,		// 173
	24,		// 174
	24,		// 175
	24,		// 176
	24,		// 177
	24,		// 178
	24,		// 179
	24,		// 180
	25,		// 181
	25,		// 182
	25,		// 183
	25,		// 184
	25,		// 185
	25,		// 186
	25,		// 187
	25,		// 188
	25,		// 189
	26,		// 190
	26,		// 191
	26,		// 192
	26,		// 193
	26,		// 194
	26,		// 195
	26,		// 196
	26,		// 197
	26,		// 198
	26,		// 199
	27,		// 200
	27,		// 201
	27,		// 202
	27,		// 203
	27,		// 204
	27,		// 205
	27,		// 206
	27,		// 207
	27,		// 208
	27,		// 209
	27,		// 210
	28,		// 211
	28,		// 212
	28,		// 213
	28,		// 214
	28,		// 215
	28,		// 216
	28,		// 217
	28,		// 218
	28,		// 219
	28,		// 220
	29,		// 221
	29,		// 222
	29,		// 223
	29,		// 224
	29,		// 225
	29,		// 226
	29,		// 227
	29,		// 228
	29,		// 229
	29,		// 230
	29,		// 231
	29,		// 232
	30,		// 233
	30,		// 234
	30,		// 235
	30,		// 236
	30,		// 237
	30,		// 238
	30,		// 239
	30,		// 240
	30,		// 241
	30,		// 242
	30,		// 243
	31,		// 244
	31,		// 245
	31,		// 246
	31,		// 247
	31,		// 248
	31,		// 249
	31,		// 250
	31,		// 251
	31,		// 252
	31,		// 253
	31,		// 254
};


